# Copyright (c) 2021, NVIDIA CORPORATION.  All rights reserved.
#
# NVIDIA CORPORATION and its licensors retain all intellectual property
# and proprietary rights in and to this software, related documentation
# and any modifications thereto.  Any use, reproduction, disclosure or
# distribution of this software and related documentation without an express
# license agreement from NVIDIA CORPORATION is strictly prohibited.
import imageio
import os
import numpy as np

import utils
from pyglet.window import key


class VideoRecorder(object):
    def __init__(self,
                 root_dir,
                 benchmark='dm_suite',
                 height=256,
                 width=256,
                 camera_id=0,
                 fps=30):
        self.save_dir = utils.make_dir(root_dir, 'video') if root_dir else None
        self.benchmark = benchmark
        self.height = height
        self.width = width
        self.camera_id = camera_id
        self.fps = fps
        self.frames = []

    def init(self, enabled=True):
        self.frames = []
        self.enabled = self.save_dir is not None and enabled

    def record(self, env):
        if self.enabled:
            if self.benchmark == 'dm_suite':
                frame = env.render(mode='rgb_array',
                                   height=self.height,
                                   width=self.width,
                                   camera_id=self.camera_id)
            else:
                # Otherwise its metaworld.
                frame = env.render(offscreen=True, camera_name="corner")
            self.frames.append(frame)

    def save(self, file_name):
        if self.enabled:
            path = os.path.join(self.save_dir, file_name)
            imageio.mimsave(path, self.frames, fps=self.fps)
